/*
 * NullEnumerator.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Collections;
using System.Collections.Generic;
using Eagle._Attributes;

namespace Eagle._Components.Private
{
    [ObjectId("005a09b4-f7b8-476f-9f87-aa409f868710")]
    internal sealed class NullEnumerator<T> : IEnumerator<T>
    {
        #region Private Data
        private bool strict;
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region Public Constructors
        public NullEnumerator()
            : this(true)
        {
            // do nothing.
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public NullEnumerator(bool strict)
        {
            this.strict = strict;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region IDisposable Members
        public void Dispose()
        {
            // do nothing.
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region IEnumerator<T> Members
        public T Current
        {
            get
            {
                //
                // NOTE: Technically, we are before the first element (i.e. our
                //       MoveNext does nothing and always returns false).  Therefore,
                //       in "strict" mode, we throw an exception here; otherwise, we
                //       simply return null.
                //
                if (strict)
                    throw new InvalidOperationException();
                else
                    return default(T);
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region IEnumerator Members
        object IEnumerator.Current
        {
            get { return ((IEnumerator<T>)this).Current; }
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public bool MoveNext()
        {
            return false;
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public void Reset()
        {
            // do nothing.
        }
        #endregion
    }
}

